import * as acorn from 'acorn';
import jsx from 'acorn-jsx';
import { extend } from 'acorn-jsx-walk';
import * as walk from 'acorn-walk';

extend(walk.base);
/**
 * Create function to extract selectors from jsx code
 *
 * @param options - acorn options
 * @returns function to extract selectors from jsx code
 *
 * @public
 */
function purgeFromJsx(options) {
    return (content) => {
        // Will be filled during walk
        const state = { selectors: [] };
        // Parse and walk any JSXElement
        walk.recursive(acorn.Parser.extend(jsx()).parse(content, options || { ecmaVersion: "latest" }), state, {
            JSXOpeningElement(acornNode, state, callback) {
                var _a;
                const node = acornNode;
                const nameState = {};
                callback(node.name, nameState);
                if (nameState.text) {
                    (_a = state.selectors) === null || _a === void 0 ? void 0 : _a.push(nameState.text);
                }
                for (let i = 0; i < node.attributes.length; ++i) {
                    callback(node.attributes[i], state);
                }
            },
            JSXAttribute(acornNode, state, callback) {
                var _a;
                const node = acornNode;
                if (!node.value) {
                    return;
                }
                const nameState = {};
                callback(node.name, nameState);
                // node.name is id or className
                switch (nameState.text) {
                    case "id":
                    case "className":
                        {
                            // Get text in node.value
                            const valueState = {};
                            callback(node.value, valueState);
                            // node.value is not empty
                            if (valueState.text) {
                                (_a = state.selectors) === null || _a === void 0 ? void 0 : _a.push(...valueState.text.split(" "));
                            }
                        }
                        break;
                }
            },
            JSXIdentifier(acornNode, state) {
                const node = acornNode;
                state.text = node.name;
            },
            JSXNamespacedName(acornNode, state) {
                const node = acornNode;
                state.text = node.namespace.name + ":" + node.name.name;
            },
            // Only handle Literal for now, not JSXExpressionContainer | JSXElement
            Literal(acornNode, state) {
                const node = acornNode;
                if (typeof node.value === "string") {
                    state.text = node.value;
                }
            },
        }, { ...walk.base });
        return state.selectors || [];
    };
}

export { purgeFromJsx as default };
